//
// ActiveConnection
//

// AO Reader and Writer for a socket

#include "ActiveConnection.h"
#include "ACObservers.h"


CSocketRead::CSocketRead(RSocket& aSocket, MSockReadObserver* aObserver, TInt aXfrLen) 
: CActive(CActive::EPriorityStandard)
, iSocket(aSocket)
, iBuffer()
, iObserver(aObserver) 
, iLen(aXfrLen) 
{
}


void CSocketRead::ConstructL() {
    User::LeaveIfNull(iObserver);
    CActiveScheduler::Add(this); 
}

CSocketRead* CSocketRead::NewL(RSocket& aSocket, MSockReadObserver* aObserver) {
    CSocketRead* self= CSocketRead::NewLC(aSocket, aObserver);
    CleanupStack::Pop();
    return self;

}

CSocketRead* CSocketRead::NewLC(RSocket& aSocket, MSockReadObserver* aObserver) {
    CSocketRead* self=new (ELeave) CSocketRead(aSocket, aObserver);
    CleanupStack::PushL(self);
    self->ConstructL();
    return self;
}

CSocketRead::~CSocketRead() {
    Cancel();
}

    
void CSocketRead::DoCancel() {

    iSocket.CancelRecv();
}


void CSocketRead::RunL() {

    TBool recv_again = ETrue;

    switch (iStatus.Int()) {
    case KErrNone:
        iObserver->OnReceiveL(iBuffer);
	break;
    default:
	recv_again = iObserver->OnReadErrorL(iStatus);
	break;
    }

    if (recv_again) {
	Receive();
    }
}


void CSocketRead::Receive() {

    iSocket.RecvOneOrMore(iBuffer, 0, iStatus, iLen);

    SetActive();
}


CSocketWrite::CSocketWrite(RSocket& aSocket) 
: CActive(CActive::EPriorityStandard)
, iSocket(aSocket)
, iRequests(_FOFF(CSocketWrite::CRequest, ilink))
{
}


void CSocketWrite::ConstructL() {
    CActiveScheduler::Add(this); 
}

CSocketWrite* CSocketWrite::NewL(RSocket& aSocket) {
    CSocketWrite* self= CSocketWrite::NewLC(aSocket);
    CleanupStack::Pop();
    return self;

}

CSocketWrite* CSocketWrite::NewLC(RSocket& aSocket) {
    CSocketWrite* self=new (ELeave) CSocketWrite(aSocket);
    CleanupStack::PushL(self);
    self->ConstructL();
    return self;
}

CSocketWrite::~CSocketWrite() {
    Cancel();

    TSglQueIter<CRequest> iterator(iRequests);
    CRequest* cursor = NULL;
    while ((cursor = iterator++) != NULL) {
	delete cursor;
    }
}

    
void CSocketWrite::DoCancel() {

    iSocket.CancelWrite();
}


void CSocketWrite::RunL() {
    
    CRequest* completed = iRequests.First();
    iRequests.Remove(*completed);
    delete completed;

    if (!iRequests.IsEmpty()) {
	IssueWrite();
    }
}


void CSocketWrite::IssueWrite() {
    iSocket.Write((*iRequests.First()).Buffer(), iStatus);

    SetActive();
}


void CSocketWrite::WriteL(const TDesC8& aBuf) {

    CRequest* iRequest = CRequest::NewL(aBuf);

    iRequests.AddLast(*iRequest);

    if (!IsActive()) {
	IssueWrite();
    }
}


CSocketWrite::CRequest* CSocketWrite::CRequest::NewL(const TDesC8& aBuf) {
    CRequest* self= CRequest::NewLC(aBuf);
    CleanupStack::Pop();
    return self;

}

CSocketWrite::CRequest* CSocketWrite::CRequest::NewLC(const TDesC8& aBuf) {
    CRequest* self=new (ELeave) CRequest();
    CleanupStack::PushL(self);
    self->ConstructL(aBuf);
    return self;
}

void CSocketWrite::CRequest::ConstructL(const TDesC8& aBuf) {
    
    iBuffer = HBufC::NewL(aBuf.Length());
    iBuffer->Des().Copy(aBuf);
}

CSocketWrite::CRequest::~CRequest() {
    delete iBuffer;
}

