//
// JTAppUI.h
//

// JTelnet's application UI class.

#if !defined(_JTAPPUI_H_)
#define _JTAPPUI_H_

#include <eikappui.h>

#include "JTelnet.h"
#include "Terminals.h"


class CJTelnetApp;
class CJTDocument;
class CHelpTask;

class CJTAppUi : public CEikAppUi 
	       , public MTerminalObserver {
public:
    // Destruction.
    ~CJTAppUi();

    // Virtual, defined by CEikAppUi; replaces the implementation 
    // provided by CEikAppUi.
    void ConstructL();

private:

    // Virtual, defined by CEikAppUi; ; empty implementation
    // provided by CEikAppUi; full implementation provided
    // by this class. 
    void  HandleCommandL(TInt aCommand);

    // Virtual, defined by CEikAppUi; replaces the implementation
    // provided by CEikAppUi.
    TBool ProcessCommandParametersL(TApaCommand aCommand,
				    TFileName& aDocumentName,
								const TDesC& aTail
							   );
    // Virtual, defined by CEikAppUi; ; empty implementation
    // provided by CEikAppUi; full implementation provided
    // by this class. 
    void  HandleModelChangeL();

    // Virtual, defined by CEikAppUi: empty implementation
    // provided by CEikAppUi; full implementation provided
    // by this class
    void DynInitMenuPaneL(TInt aMenuId,CEikMenuPane* aMenuPane);
    // Virtual, defined by CEikAppUi:  empty implementation
    // provided by CEikAppUi; full implementation provided
    // by this class
    void OpenFileL(const TDesC& aFileName);
    void CreateFileL(const TDesC& aFileName);


    // -- MTerminalObserver
    virtual void NotifyPhaseChange( MTerminal::EConnectPhase prev
	, MTerminal::EConnectPhase current
				  );

    // -- MConnectionObserver
    virtual void OnServerStart(TRequestStatus aStatus);
    virtual void OnAddressResolved(TRequestStatus aStatus);
    virtual void OnConnect(TRequestStatus aStatus);
    virtual void OnDisconnect(TRequestStatus aStatus);

private:
    // Member functions defined by this class
    void OnCmdFileNewL();
    void OnCmdFileOpenL();
    void OnCmdFileSaveAsL();
    void OnCmdFileSaveL();
    void OnCmdExit();
    void OnCmdUpdateData();
    void OnCmdToggleShowToolbar();
    void OnCmdConnectL();
    void OnCmdServerDetailsL();
    void OnCmdAboutL();
    void OnCmdDisconnectNetL();
//    void OnCmdZoomIn();
//    void OnCmdZoomOut();
    
    void ResetViewL();
    void DrawViewL();
    void UpdateFileNameLabelL();

    void ToolButtonSetTextL(TInt aId, TInt aResourceId);

    enum EBtnVisibility {
	EDimmed
      , ENormal
    };
    void ToolButtonSetDimmed(TInt aId, EBtnVisibility aDimmed);

    CJTelnetApp* JTApp() const { return STATIC_CAST(CJTelnetApp*, Application()); }
    CJTDocument* JTDoc() const { return STATIC_CAST(CJTDocument*, Document()); }

private:
    // Data members defined by this class.
    CTTY*      iContainer;     // The container control (ie the view)
    MTerminal* Terminal() const { return STATIC_CAST(MTerminal*, iContainer); }

    TBool      iShowToolBar;
    CHelpTask* iHelp;		// help file manager.

    TBool      iExitingAsync;	// used for orderly shutdown...

    MTerminal::EConnectPhase iPhase;

    TZoomFactor iAppZoom;
};

#endif