//
// Terminals.h
//

// Terminal controls that implement policy on top of the NVT.

#ifndef __JTTERMINALS_H__
#define __JTTERMINALS_H__

#include <coecntrl.h>
#include "ACObservers.h"
#include "NVTObserver.h"

class CEikGlobalTextEditor;
class CNVT;

// a class for the app as terminal client...
class MTerminal {
public:

    enum EConnectPhase {
	ETermDisconnected
      , ETermConnecting
      , ETermConnected
      , ETermDisconnecting
    };

    virtual EConnectPhase Phase() const = 0;
    
    // Phase() helpers...
    TBool IsConnected()    const { return Phase() == ETermConnected; }
    TBool IsDisconnected() const { return Phase() == ETermDisconnected; }
    TBool IsActive()       const { return !IsDisconnected(); }

    enum TDisconnectType {
	EDisconnectNormal
      , EDisconnectNet
    };

    enum {
	EDefaultPort = 23
    };

    // only leave on error - use MTerminalObserver to determine network operation success
    virtual void ConnectL(const TDesC& aAddr, const TUint aPort) = 0;
    virtual void DisconnectL(TDisconnectType aType) = 0;


    enum TClipboardFunc {
	ECopy,
	EPaste
    };

    virtual void ClipboardL(TClipboardFunc aClipboardFunc) = 0;

    virtual void SelectAllL() = 0;


    enum TZoomLevel {
	EZoomStateOne   = 660
      , EZoomStateTwo   = 800
      , KZoomStateThree = TZoomFactor::EZoomOneToOne
      , KZoomStateFour  = 1500
    };

    virtual void SetZoomLevelL(TZoomLevel) = 0;
    virtual void SetZoomFactorL(TZoomFactor*) = 0;

};

class MTerminalObserver : public MConnectionObserver {
public:
    virtual void NotifyPhaseChange( MTerminal::EConnectPhase prev
				  , MTerminal::EConnectPhase current
				  ) = 0;
};


class CTTY : public CCoeControl
	   , public MCoeControlObserver 
	   , public MNVTObserver 
	   , public MTerminal {

    CTTY();
    virtual void ConstructL(const TRect& aRect, MTerminalObserver* aObserver);

public:

    static CTTY* NewLC(const TRect& aRect, MTerminalObserver* aObserver);
    virtual ~CTTY();

    // CCoeControl
    virtual void Draw(const TRect&) const {};	//empty
    TInt         CountComponentControls() const { return 1; }
    CCoeControl* ComponentControl(TInt aIndex) const;
    TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType);
    virtual void SizeChangedL();

    void ActivateL();
    virtual void FocusChanged(TDrawNow aDrawNow);

    // MCoeControlObserver
    virtual void HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType);

    // MTerminal
    virtual EConnectPhase Phase() const;
    virtual void ConnectL(const TDesC& aAddr, const TUint aPort);
    virtual void DisconnectL(TDisconnectType aType);
    virtual void ClipboardL(TClipboardFunc aClipboardFunc);
    virtual void SelectAllL();
    virtual void SetZoomLevelL(TZoomLevel);
    virtual void SetZoomFactorL(TZoomFactor*);

    // MNVTObserver
    virtual void OnBufferUpdateL(const TChar aChar);
    virtual void OnBufferUpdateL(const TDesC& aBuf);
    virtual void OnServerStart(TRequestStatus aStatus);
    virtual void OnAddressResolved(TRequestStatus aStatus);
    virtual void OnConnect(TRequestStatus aStatus);
    virtual void OnDisconnect(TRequestStatus aStatus);

    void PhaseChange(EConnectPhase prev, EConnectPhase current);


private:

    static const TUid KUidBufferLimit;

    CEikGlobalTextEditor* iEdwin;
    CNVT*	          iNVT;

    MTerminalObserver*    iTermObserver;

    TInt                  iBufferLimit;

    EConnectPhase iPhase;
};



#endif // __JTTERMINALS_H__